import { Component, ViewChild } from "@angular/core";

import { FixedVarResultsComponent } from "../fixedvar-results/fixedvar-results.component";
import { JetTrajectoryChartComponent } from "../jet-trajectory-chart/jet-trajectory-chart.component";

@Component({
    selector: "jet-results",
    templateUrl: "./jet-results.component.html",
    styleUrls: [
        "./jet-results.component.scss"
    ]
})
export class JetResultsComponent extends FixedVarResultsComponent {

    /** graphique de trajectoire */
    @ViewChild(JetTrajectoryChartComponent)
    private jetTrajectoryChartComponent: JetTrajectoryChartComponent;

    public get hasResults(): boolean {
        return (
            (this._fixedResults !== undefined && this._fixedResults.hasResults)
            ||
            (this._varResults !== undefined && this._varResults.hasResults)
        );
    }

    public updateView() {
        if (this.jetTrajectoryChartComponent) {
            this.jetTrajectoryChartComponent.results = undefined;
        }
        super.updateView();
    }

    /**
     * met à jour l'affichage des résultats
     * @returns true si les résultats ont pu être mis à jour
     */
    protected updateResults() {
        const superUpdated = super.updateResults();

        let trajectoryChartUpdated: boolean;
        trajectoryChartUpdated = this.jetTrajectoryChartComponent !== undefined;

        if (trajectoryChartUpdated) {
            // draw chart whether params are variating or not,
            // hence different Results object for each case
            if (this._varResults && this._varResults.hasResults) {
                this.jetTrajectoryChartComponent.results = this._varResults;
            } else {
                this.jetTrajectoryChartComponent.results = this._fixedResults;
            }
            this.jetTrajectoryChartComponent.updateView();
        }

        return superUpdated && trajectoryChartUpdated;
    }
}
