import { Component, Input, OnInit } from "@angular/core";
import { Router } from "@angular/router";

import { SelectEntry } from "../../formulaire/elements/select-entry";
import { FormulaireService } from "../../services/formulaire.service";
import { FieldsetContainer } from "../../formulaire/elements/fieldset-container";
import { SelectFieldModel } from "../../formulaire/elements/select-field-model";

@Component({
    selector: "select-model-field-line",
    templateUrl: "./select-model-field-line.component.html",
    styleUrls: [
        "./select-model-field-line.component.scss"
    ]
})
export class SelectModelFieldLineComponent implements OnInit {
    @Input()
    private _select: SelectFieldModel;

    public constructor(
        private _formService: FormulaireService,
        private router: Router
    ) {}

    public get selectId() {
        return this._select.id;
    }

    public get entries(): SelectEntry[] {
        if (! this._select) {
            return [];
        }
        return this._select.entries;
    }

    public get modelUid() {
        return this.selectedValue ? this.selectedValue.id : "";
    }

    public get isValid(): boolean {
        return (this._select.getValue() !== undefined);
    }

    // called every time we navigate to the module
    ngOnInit(): void {
        this._select.updateEntries();
    }

    /**
     * Label d'une entrée du select des modèles de cloisons
     */
    public selectItemLabel(entry: SelectEntry): string {
        if (entry && entry.id) {
            const cloisonsForm = this._formService.getFormulaireFromNubId(entry.id);
            const name = cloisonsForm.calculatorName;
            return name;
        }
    }

    public get selectedValue(): SelectEntry {
        return this._select.getValue();
    }

    public set selectedValue(v: SelectEntry) {
        this._select.setValue(v);
    }

    public get label() {
        if (this._select) {
            return this._select.label;
        } else {
            return "";
        }
    }

    /**
     * Creates a new Nub of type _select.calcType, to be used as a model
     * when no other is available
     */
    public createModel() {
        this._formService.createFormulaire(this._select.calcType).then(f => {
            this.router.navigate(["/calculator", f.uid]);
            return f;
        }).then(f => {
            // on ajoute un ouvrage aux modulex de type "parallèle"
            for (const e of f.allFormElements) {
                if (e instanceof FieldsetContainer) {
                    e.addFromTemplate(0);
                    break;
                }
            }
        });
    }
}
