import { Component, Output, EventEmitter } from "@angular/core";

import { I18nService } from "../../services/internationalisation.service";
import { fv, longestVarParam } from "../../util";
import { MultiDimensionResults } from "../../results/multidimension-results";

@Component({
    selector: "variable-results-selector",
    templateUrl: "./variable-results-selector.component.html",
    styleUrls: [
        "./variable-results-selector.component.scss"
    ]
})
export class VariableResultsSelectorComponent {

    /** résultats non mis en forme */
    private _results: MultiDimensionResults;

    private _selectedValue: number;

    /** size of the longest variable value */
    private size = 0;

    /** inferred extended values list for each variating parameter */
    private varValues = [];

    @Output()
    protected indexChange = new EventEmitter();

    constructor(
        protected intlService: I18nService,
    ) {
        this._selectedValue = 0;
    }

    public set results(r: MultiDimensionResults) {
        this._results = r;

        if (this._results) {
            // pre-extract variable parameters values
            this.varValues = [];
            // find longest list
            const lvp = longestVarParam(this._results.variatedParameters);
            this.size = lvp.size;
            // get extended values lists for each variable parameter
            for (const v of this._results.variatedParameters) {
                const vv = [];
                const iter = v.getExtendedValuesIterator(this.size);
                while (iter.hasNext) {
                    const nv = iter.next();
                    vv.push(fv(nv.value));
                }
                this.varValues.push(vv);
            }
        }
    }

    public get hasVariableResults(): boolean {
        return (
            this._results
            && this._results.hasResults
            && this._results.variatedParameters.length > 0
        );
    }

    public get entries(): number[] {
        const ret: number[] = [];
        for (let i = 0; i < this.size; i++) {
            ret.push(i);
        }
        return ret;
    }

    protected entryLabel(index: number): string {
        const kv = [];
        for (let i = 0; i < this.varValues.length; i++) {
            const vv = this.varValues[i];
            const vp = this._results.variatedParameters[i];
            let symbol = vp.symbol;
            // is vp a parameter of a child Nub ?
            if (
                vp.paramDefinition.parentNub
                && vp.paramDefinition.parentNub !== vp.paramDefinition.originNub
            ) {
                const pos = vp.paramDefinition.parentNub.findPositionInParent() + 1;
                symbol = this.intlService.localizeText("INFO_LIB_RADIER_N_COURT") + pos + "_" + symbol;
            }
            kv.push(`${symbol} = ${vv[index]}`);
        }
        return kv.join(", ");
    }

    public get selectedValue(): number {
        return this._selectedValue;
    }

    public set selectedValue(v: number) {
        this._results.variableIndex = v;
        this.indexChange.emit();
    }

    public get label() {
        return this.intlService.localizeText("INFO_PARAMFIELD_BOUNDARY_CONDITIONS");
    }
}
