import { Nub, ParamDefinition, Result, SectionParametree, acSection } from "jalhyd";

import { FormulaireSection } from "./form-section";
import { SectionResults } from "../../results/section-results";
import { ParamRadioConfig, NgParameter } from "../elements/ngparam";
import { CalculatorResults } from "../../results/calculator-results";

export class FormulaireSectionParametree extends FormulaireSection {

    /**
     * résultats de section paramétrée
     */
    private _sectionResults: SectionResults;

    public constructor() {
        super();
        this._sectionResults = new SectionResults();
    }

    protected compute() {
        this.runNubCalc(this.currentNub);
        this.reaffectResultComponents();
    }

    protected runNubCalc(nub: Nub, computedParam?: ParamDefinition): Result {
        return nub.CalcSerie();
    }

    protected reaffectResultComponents() {
        const sectNub: SectionParametree = this.currentNub as SectionParametree;
        const sect: acSection = sectNub.section;
        this._sectionResults.section = sect;

        const varParams = this.getSectionVariatedParameters();
        if (varParams.length > 0) {
            // résultats variés avec tous les résultats complémentaires
            this._varResults.variatedParameters = varParams;
            this._varResults.result = sectNub.result;
            this._varResults.update();
        } else {
            // résultats de section (avec le graphique de section)
            this._sectionResults.result = sectNub.result;
            // résultats complémentaires des paramètres fixés
            this.addSectionFixedParameters();
            this.fixedResults.result = sectNub.result;
        }

    }

    public resetFormResults() {
        this._fixedResults.reset();
        this._varResults.reset();
        this._sectionResults.reset();
    }

    public addSectionFixedParameters() {
        for (const p of this.getDisplayedParamListFromState(ParamRadioConfig.FIX)) {
            this._fixedResults.addFixedParameter(p);
        }
    }

    public get hasResults(): boolean {
        return (this._fixedResults !== undefined && this._fixedResults.hasResults)
            || (this._varResults !== undefined && this._varResults.hasResults)
            || (this._sectionResults !== undefined && this._sectionResults.hasResults);
    }

    public get results(): CalculatorResults[] {
        const res: CalculatorResults[] = [];
        // ensure help links are propagated
        this._fixedResults.helpLinks = this.helpLinks;
        this._varResults.helpLinks = this.helpLinks;
        this._sectionResults.helpLinks = this.helpLinks;
        res.push(this._fixedResults);
        res.push(this._varResults);
        res.push(this._sectionResults);
        return res;
    }

    public getSectionVariatedParameters(): NgParameter[] {
        return this.getDisplayedParamListFromState(ParamRadioConfig.VAR);
    }
}
