import { NgParameter } from "./formulaire/elements/ngparam";
import { ServiceFactory } from "./services/service-factory";

import { formattedValue, Nub, ParamDefinition, ParamValues } from "jalhyd";

export function logObject(obj: {}, m?: string) {
    // évite le message "Value below was evaluated just now" dans le debugger de Chrome
    if (m === undefined) {
        console.log(JSON.stringify(obj));
    } else {
        console.log(m + " " + JSON.stringify(obj));
    }
}

export function isNumber(s: string): boolean {
    return Number(s) !== NaN;
}

/**
 * Proxy to jaLhyd.formattedValue() using number of digits from app preferences
 */
export function fv(p: NgParameter | number): string {
    let value: number;
    if (p instanceof NgParameter) {
        value = p.getValue();
    } else if (typeof p === "number") {
        value = p;
    }
    const nDigits = ServiceFactory.instance.applicationSetupService.displayPrecision;

    return formattedValue(value, nDigits);
}

/**
 * Trick to decode HTML entities in a string
 * https://stackoverflow.com/a/7394787/5986614
 * @param html string containing HTML entities, like &nbsp;
 */
export function decodeHtml(html: string): string {
    const txt = document.createElement("textarea");
    txt.innerHTML = html;
    return txt.value;
}

/**
 * Given a list of variated NgParameter, returns the parameter having the most
 * values, its index in the list, and the number of values it contains
 * @param varParams
 */
export function longestVarParam(varParams: NgParameter[]): { param: NgParameter, index: number, size: number } {
    const variated: { param: ParamDefinition, values: ParamValues }[] = [];
    for (const vp of varParams) {
        variated.push({
            param: vp.paramDefinition,
            values: vp.paramDefinition.paramValues
        });
    }
    const { size, longest, minLinkedResultParam } = Nub.findVariatedSize(variated);
    let realSize = size;
    // if at least one linked variated result was found
    if (minLinkedResultParam !== undefined) {
        // if the size limited by linked variated results is shorter
        // than the size of the longest variating element, limit it
        if (minLinkedResultParam.values.valuesIterator.count() < realSize) {
            realSize = minLinkedResultParam.values.valuesIterator.count();
        }
    }
    return {
        param: varParams[longest],
        index: longest,
        size: realSize
    };
}
